@@ -1,23 +1,29 @@
 # encoding=UTF-8
 
-# Copyright © 2009-2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009-2018 Jakub Wilk <jwilk@jwilk.net>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 dated June, 1991.
+# This file is part of didjvu.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# General Public License for more details.
+# didjvu is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# didjvu is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 
-'''command line interface for *didjvu*'''
+'''
+didjvu's command-line interface
+'''
+
+import functools
 
 from . import utils
 
 try:
     import argparse
-except ImportError as ex:  # <no-coverage>
+except ImportError as ex:  # no coverage
     utils.enhance_import_error(ex,
         'argparse',
         'python-argparse',
@@ -25,12 +31,9 @@ except ImportError as ex:  # <no-coverage>
     )
     raise
 
-from . import djvu_extra as djvu
-from . import version as version_module
-try:
-    from . import xmp
-except ImportError as xmp_import_error:  # <no-coverage>
-    xmp = None
+from . import djvu_support as djvu
+from . import version
+from . import xmp
 
 def range_int(x, y, typename):
     class rint(int):
@@ -80,7 +83,7 @@ def slice_type(max_slices=djvu.IW44_N_SLICES_MAX):
 def get_slice_repr(lst):
     def fold(lst, obj):
         return lst + [obj - sum(lst)]
-    plus_lst = reduce(fold, lst[1:], lst[:1])
+    plus_lst = functools.reduce(fold, lst[1:], lst[:1])
     return '+'.join(map(str, plus_lst))
 
 class intact(object):
@@ -122,7 +125,7 @@ def _get_method_params_help(methods):
 class ArgumentParser(argparse.ArgumentParser):
 
     class defaults:
-        pageid_template = '{base-ext}.djvu'
+        page_id_template = '{base-ext}.djvu'
         pages_per_dict = 1
         dpi = None
         fg_slices = [100]
@@ -134,8 +137,7 @@ class ArgumentParser(argparse.ArgumentParser):
 
     def __init__(self, methods, default_method):
         argparse.ArgumentParser.__init__(self, formatter_class=argparse.RawDescriptionHelpFormatter)
-        version = version_module.get_software_agent()
-        self.add_argument('--version', action='version', version=version, help='show version information and exit')
+        self.add_argument('--version', action=version.VersionAction)
         p_separate = self.add_subparser('separate', help='generate masks for images')
         p_encode = self.add_subparser('encode', help='convert images to single-page DjVu documents')
         p_bundle = self.add_subparser('bundle', help='convert images to bundled multi-page DjVu document')
@@ -146,13 +148,17 @@ class ArgumentParser(argparse.ArgumentParser):
             p.add_argument('-o', '--output', metavar='FILE', help='output filename')
             if p is p_bundle:
                 p.add_argument(
-                    '--pageid-template', metavar='TEMPLATE', default=default.pageid_template,
-                    help='naming scheme for page identifiers (default: "{template}")'.format(template=default.pageid_template)
+                    '--page-id-template', metavar='TEMPLATE', default=default.page_id_template,
+                    help='naming scheme for page identifiers (default: "{template}")'.format(template=default.page_id_template)
                 )
+                p.add_argument(
+                    '--pageid-template', dest='page_id_template', metavar='TEMPLATE',
+                    help=argparse.SUPPRESS
+                )  # obsolete alias
             else:
                 p.add_argument(
                     '--output-template', metavar='TEMPLATE',
-                    help='naming scheme for output file (e.g. "{template}")'.format(template=default.pageid_template)
+                    help='naming scheme for output file (e.g. "{template}")'.format(template=default.page_id_template)
                 )
             p.add_argument('--losslevel', dest='loss_level', type=losslevel_type, help=argparse.SUPPRESS)
             p.add_argument(
@@ -314,8 +320,8 @@ class ArgumentParser(argparse.ArgumentParser):
         o.method = self.__methods[o.method]
         o.params = self._parse_params(o)
         try:
-            if not xmp and o.xmp:
-                raise xmp_import_error
+            if o.xmp and not xmp.backend:
+                raise xmp.import_error
         except AttributeError:
             pass
         return action(o)
