@@ -1,37 +1,39 @@
 # encoding=UTF-8
 
-# Copyright © 2009-2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009-2018 Jakub Wilk <jwilk@jwilk.net>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 dated June, 1991.
+# This file is part of didjvu.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# General Public License for more details.
+# didjvu is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# didjvu is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
+
+'''
+didjvu core
+'''
 
 from __future__ import print_function
 
 import itertools
 import logging
 import os
-import re
 import sys
 
 from . import cli
-from . import djvu_extra as djvu
+from . import djvu_support as djvu
 from . import filetype
 from . import fs
-from . import gamera_extra as gamera
+from . import gamera_support as gamera
 from . import ipc
 from . import templates
 from . import temporary
-
-try:
-    from . import xmp
-except ImportError:
-    xmp = None
+from . import utils
+from . import xmp
 
 logger = None
 
@@ -58,7 +60,7 @@ def setup_logging():
 def error(message, *args, **kwargs):
     if args or kwargs:
         message = message.format(*args, **kwargs)
-    print('didjvu: error: ' + message, file=sys.stderr)
+    print('didjvu: error: {msg}'.format(msg=message), file=sys.stderr)
     sys.exit(1)
 
 def parallel_for(o, f, *iterables):
@@ -180,24 +182,6 @@ def generate_mask(filename, image, method, params):
     else:
         return gamera.load_image(filename)
 
-_pageid_chars = re.compile('^[A-Za-z0-9_+.-]+$').match
-
-def check_pageid_sanity(pageid):
-    if not _pageid_chars(pageid):
-        raise ValueError('pageid must consist only of lowercase ASCII letters, digits, _, +, - and dot.')
-    if pageid[:1] in ('.', '+', '-'):
-        raise ValueError('pageid cannot start with +, - or a dot.')
-    if '..' in pageid:
-        raise ValueError('pageid cannot contain two consecutive dots.')
-    assert pageid == os.path.basename(pageid)
-    if pageid.endswith('.djvu'):
-        return pageid
-    else:
-        raise ValueError('pageid must end with the .djvu extension.')
-
-class namespace():
-    pass
-
 class main():
 
     compression_info_template = (
@@ -233,11 +217,11 @@ class main():
 
     def check_multi_output(self, o):
         self.check_common(o)
-        pageid_memo = {}
+        page_id_memo = {}
         if o.output is None:
             if o.output_template is not None:
                 o.output = [
-                    templates.expand(o.output_template, f, n, pageid_memo)
+                    templates.expand(o.output_template, f, n, page_id_memo)
                     for n, f in enumerate(o.input)
                 ]
                 o.xmp_output = [s + '.xmp' if o.xmp else None for s in o.output]
@@ -271,8 +255,8 @@ class main():
             check_tty()
         else:
             filename = o.output
-            o.output = [file(filename, 'wb')]
-            o.xmp_output = [file(filename + '.xmp', 'wb')] if o.xmp else [None]
+            o.output = [open(filename, 'wb')]
+            o.xmp_output = [open(filename + '.xmp', 'wb')] if o.xmp else [None]
         assert len(o.output) == len(o.xmp_output) == 1
 
     def encode(self, o):
@@ -383,12 +367,15 @@ class main():
         with temporary.directory() as tmpdir:
             bytes_in = 0
             component_filenames = []
-            pageid_memo = {}
+            page_id_memo = {}
             for page, (input, mask) in enumerate(zip(o.input, o.masks)):
                 bytes_in += os.path.getsize(input)
-                pageid = templates.expand(o.pageid_template, input, page, pageid_memo)
-                check_pageid_sanity(pageid)
-                component_filenames += os.path.join(tmpdir, pageid),
+                page_id = templates.expand(o.page_id_template, input, page, page_id_memo)
+                try:
+                    djvu.validate_page_id(page_id)
+                except ValueError as exc:
+                    error(exc)
+                component_filenames += [os.path.join(tmpdir, page_id)]
             parallel_for(o, self._bundle_simple_page, o.input, o.masks, component_filenames)
             logger.info('bundling')
             djvu_file = djvu.bundle_djvu(*component_filenames)
@@ -419,7 +406,7 @@ class main():
         page.djvu = image_to_djvu(width, height, image, mask, options=o)
         image = mask = None
         page.sjbz = djvu.Multichunk(width, height, dpi, sjbz=page.djvu['sjbz'])
-        page.sjbz_symlink = os.path.join(minidjvu_in_dir, page.pageid)
+        page.sjbz_symlink = os.path.join(minidjvu_in_dir, page.page_id)
         os.symlink(page.sjbz.save().name, page.sjbz_symlink)
 
     def bundle_complex(self, o):
@@ -428,16 +415,16 @@ class main():
             bytes_in = 0
             pixels = [0]
             page_info = []
-            pageid_memo = {}
+            page_id_memo = {}
             for pageno, (image_filename, mask_filename) in enumerate(zip(o.input, o.masks)):
-                page = namespace()
-                page_info += page,
+                page = utils.namespace()
+                page_info += [page]
                 bytes_in += os.path.getsize(image_filename)
-                page.pageid = templates.expand(o.pageid_template, image_filename, pageno, pageid_memo)
+                page.page_id = templates.expand(o.page_id_template, image_filename, pageno, page_id_memo)
                 try:
-                    check_pageid_sanity(page.pageid)
+                    djvu.validate_page_id(page.page_id)
                 except ValueError as exc:
-                    error(str(exc))
+                    error(exc)
             del page  # quieten pyflakes
             parallel_for(o, self._bundle_complex_page,
                 page_info,
@@ -466,15 +453,15 @@ class main():
                 component_filenames = []
                 for pageno, page in enumerate(page_info):
                     if pageno % o.pages_per_dict == 0:
-                        iff_name = fs.replace_ext(page_info[pageno].pageid, 'iff')
+                        iff_name = fs.replace_ext(page_info[pageno].page_id, 'iff')
                         iff_name = os.path.join(minidjvu_out_dir, iff_name)
                         component_filenames += [iff_name]
-                    sjbz_name = os.path.join(minidjvu_out_dir, page.pageid)
+                    sjbz_name = os.path.join(minidjvu_out_dir, page.page_id)
                     component_filenames += [sjbz_name]
                     page.djvu['sjbz'] = sjbz_name
                     page.djvu['incl'] = iff_name
                     page.djvu = page.djvu.save()
-                    page.djvu_symlink = os.path.join(minidjvu_out_dir, page.pageid)
+                    page.djvu_symlink = os.path.join(minidjvu_out_dir, page.page_id)
                     os.unlink(page.djvu_symlink)
                     os.symlink(page.djvu.name, page.djvu_symlink)
                 logger.info('bundling')
@@ -489,4 +476,6 @@ class main():
         compression_info = self.compression_info_template.format(**locals())
         logger.nosy(compression_info)
 
+__all__ = ['main']
+
 # vim:ts=4 sts=4 sw=4 et
