@@ -1,15 +1,17 @@
 # encoding=UTF-8
 
-# Copyright © 2010-2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2010-2016 Jakub Wilk <jwilk@jwilk.net>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 dated June, 1991.
+# This file is part of didjvu.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# General Public License for more details.
+# didjvu is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# didjvu is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 
 '''
 filetype detection
@@ -18,7 +20,7 @@ filetype detection
 class generic(object):
 
     def __new__(self, *args, **kwargs):
-        raise NotImplementedError
+        raise NotImplementedError  # no coverage
 
     @classmethod
     def like(cls, other):
