@@ -1,21 +1,24 @@
 # encoding=UTF-8
 
-# Copyright © 2008-2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2008-2016 Jakub Wilk <jwilk@jwilk.net>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 dated June, 1991.
+# This file is part of didjvu.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# General Public License for more details.
+# didjvu is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# didjvu is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 
 '''interprocess communication'''
 
 import errno
 import logging
 import os
+import pipes
 import re
 import signal
 import subprocess
@@ -30,21 +33,21 @@ if os.name == 'posix':
     signal.signal(signal.SIGCHLD, signal.SIG_DFL)
 
 def get_signal_names():
-    _signame_pattern = re.compile('^SIG[A-Z0-9]*$')
+    signame_pattern = re.compile('^SIG[A-Z0-9]*$')
     data = dict(
         (name, getattr(signal, name))
         for name in dir(signal)
-        if _signame_pattern.match(name)
+        if signame_pattern.match(name)
     )
     try:
         if data['SIGABRT'] == data['SIGIOT']:
             del data['SIGIOT']
-    except KeyError:  # <no-coverage>
+    except KeyError:  # no coverage
         pass
     try:
         if data['SIGCHLD'] == data['SIGCLD']:
             del data['SIGCLD']
-    except KeyError:  # <no-coverage>
+    except KeyError:  # no coverage
         pass
     return dict((no, name) for name, no in data.iteritems())
 
@@ -66,13 +69,8 @@ del get_signal_names
 # Subprocess
 # ==========
 
-def shell_escape(s, safe=re.compile('^[a-zA-Z0-9_+/=.,:%-]+$').match):
-    if safe(s):
-        return s
-    return "'{0}'".format(s.replace("'", r"'\''"))
-
-def shell_escape_list(lst):
-    return ' '.join(map(shell_escape, lst))
+def shell_escape(commandline):
+    return ' '.join(map(pipes.quote, commandline))
 
 class Subprocess(subprocess.Popen):
 
@@ -103,7 +101,7 @@ class Subprocess(subprocess.Popen):
         except KeyError:
             commandline = args[0]
         if logger.isEnabledFor(logging.DEBUG):
-            logger.debug(shell_escape_list(commandline))
+            logger.debug(shell_escape(commandline))
         self.__command = commandline[0]
         try:
             subprocess.Popen.__init__(self, *args, **kwargs)
@@ -123,30 +121,6 @@ class Subprocess(subprocess.Popen):
 
 PIPE = subprocess.PIPE
 
-# Proxy
-# =====
-
-class Proxy(object):
-
-    def __init__(self, obj, wait_fn, temporaries):
-        self._object = obj
-        self._wait_fn = wait_fn
-        self._temporaries = temporaries
-
-    def __getattribute__(self, name):
-        if name.startswith('_'):
-            return object.__getattribute__(self, name)
-        self._wait_fn()
-        self._wait_fn = int
-        return getattr(self._object, name)
-
-    def __setattr__(self, name, value):
-        if name.startswith('_'):
-            return object.__setattr__(self, name, value)
-        self._wait_fn()
-        self._wait_fn = int
-        setattr(self._object, name, value)
-
 # require()
 # =========
 
@@ -173,7 +147,6 @@ logger = logging.getLogger('didjvu.ipc')
 __all__ = [
     'CalledProcessError', 'CalledProcessInterrupted',
     'Subprocess', 'PIPE',
-    'Proxy',
 ]
 
 # vim:ts=4 sts=4 sw=4 et
