@@ -1,15 +1,17 @@
 # encoding=UTF-8
 
-# Copyright © 2010-2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2010-2018 Jakub Wilk <jwilk@jwilk.net>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 dated June, 1991.
+# This file is part of didjvu.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# General Public License for more details.
+# didjvu is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# didjvu is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 
 '''filename templates'''
 
@@ -37,8 +39,8 @@ def expand(template, name, page, memo):
             sign = -1
             base_var, offset = var.split('-', 1)
         else:
-            continue  # <no-coverage>
-            # https://bitbucket.org/ned/coveragepy/issue/198
+            continue  # no coverage
+            # https://bitbucket.org/ned/coveragepy/issues/198
         try:
             offset = sign * int(offset, 10)
         except ValueError:
@@ -62,4 +64,6 @@ def expand(template, name, page, memo):
     memo[ident] = 1
     return ident
 
+__all__ = ['expand']
+
 # vim:ts=4 sts=4 sw=4 et
