@@ -2,14 +2,16 @@
 
 # Copyright © 2012-2015 Jakub Wilk <jwilk@jwilk.net>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 dated June, 1991.
+# This file is part of didjvu.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# General Public License for more details.
+# didjvu is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# didjvu is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 
 '''XMP support (python-xmp-toolkit backend)'''
 
@@ -28,7 +30,7 @@ class MetadataBase(object):
         backend = self._backend = libxmp.XMPMeta()
         prefix = backend.register_namespace(ns.didjvu, 'didjvu')
         if prefix is None:
-            raise XmpError('Cannot register namespace for didjvu internal properties')  # <no-coverage>
+            raise XmpError('Cannot register namespace for didjvu internal properties')  # no coverage
 
     @classmethod
     def _expand_key(cls, key):
@@ -70,9 +72,9 @@ class MetadataBase(object):
                 value = str(value)
             rc = backend.set_property(namespace, key, value)
         if rc is False:
-            raise XmpError('Cannot set property')  # <no-coverage>
+            raise XmpError('Cannot set property')  # no coverage
 
-    def add_to_history(self, event, index):
+    def _add_to_history(self, event, index):
         for key, value in event.items:
             if value is None:
                 continue
@@ -86,7 +88,7 @@ class MetadataBase(object):
         if count == 0:
             self['xmpMM.History'] = []
             assert count_history() == 0
-        result = self.add_to_history(event, count + 1)
+        result = self._add_to_history(event, count + 1)
         assert count_history() == count + 1
         return result
 
