@@ -2,18 +2,20 @@
 
 # Copyright © 2015 Jakub Wilk <jwilk@jwilk.net>
 #
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 dated June, 1991.
+# This file is part of didjvu.
 #
-# This program is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-# General Public License for more details.
+# didjvu is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# didjvu is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
+# for more details.
 
-import cStringIO as io
+import io
 
-from . common import (
+from .tools import (
     assert_equal,
     interim,
 )
@@ -22,8 +24,8 @@ from lib import fs
 
 def test_copy_file():
     def t(s):
-        input_file = io.StringIO(s)
-        output_file = io.StringIO()
+        input_file = io.BytesIO(s)
+        output_file = io.BytesIO()
         length = fs.copy_file(input_file, output_file)
         assert_equal(output_file.tell(), length)
         output_file.seek(0)
